# Freeping Creaturism:
#
# this is the all important version number used by pip.
#
#
"""
Version History:

1.0.0    - 8/2/2016 initial birth as a pip package.

1.0.1    - 8/2/2016 the rst gets a make over, and doc strings
           became more pervasive.

1.0.2    - 8/4/2016 dragonegg issues. trying to include a polite 'as' wrapper
           (i.e. not a  console_script called as).

1.0.3    - 8/4/2016 travis build fixes.

1.0.4    - 8/4/2016 travis build fixes, and exception handling fixes.

1.0.5    - 8/4/2016 exit value was upsetting travis.

1.0.6    - 8/9/2016 exit codes preserved; important for configure scripts like musl libc.

1.0.7    - 8/9/2016 logical restructuring; smaller bites.

1.0.8    - 8/9/2016 test and doc subdirectories are no longer included.

1.0.9    - 8/25/2016  Python 3.0 import fixes (Will Dietz)

1.0.10   - 9/26/2016  Apple's otool just gets biggier and buggier.

1.0.11   - 9/27/2016  Improved Apple's otool fix.

1.0.12   - 10/27/2016 Common flag support.

1.0.13   - 11/05/2016  pylint spots a few mistakes.

1.0.14   - 11/10/2016  --coverage flag.

1.0.15   - 11/15/2016  pylintification complete.

1.0.16   - 11/16/2016  ooops musl points out I screwed up the exit codes AGAIN.

1.0.17   - 11/23/2016  delcypher  #16  over at travitch's place.

"""

wllvm_version = '1.0.17'
